//
//  ExplorerViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/08/10.
//

import Foundation
import Combine

final class ExplorerViewModel: ObservableObject {
    
    private let repository = ExplorerRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var explorerSections = [GetExplorerSectionResponse]()
    @Published var channelResponses = [GetRoomDetailUser]()
    
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    @Published var isLoading = false
    
    @Published var channel = ""
    
    init() {
        _channel = Published(initialValue: "")
        $channel
            .debounce(for: .seconds(0.3), scheduler: RunLoop.main)
            .sink { [unowned self] value in
                DEBUG_LOG("value: \(value)")
                if value.count > 1 {
                    UserDefaults.set(value, forKey: .searchChannel)
                    searchChannel()
                }
            }
            .store(in: &subscription)
    }
    
    func getExplorer() {
        explorerSections.removeAll()
        isLoading = true
        
        repository.getExplorer()
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                self.isLoading = false
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetExplorerResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.explorerSections.append(contentsOf: data.sections)
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
    
    private func searchChannel() {
        repository.searchChannel(channel: channel)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetRoomDetailUser]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        channelResponses.removeAll()
                        channelResponses.append(contentsOf: data)
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
            }
            .store(in: &subscription)
    }
}
