//
//  CreatorCommunityWriteViewModel.swift
//  SodaLive
//
//  Created by klaus on 2023/12/19.
//

import UIKit

import Moya
import Combine

final class CreatorCommunityWriteViewModel: ObservableObject {
    private let repository = CreatorCommunityRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var isLoading = false
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    
    @Published var content = ""
    @Published var isAdult = false
    @Published var isPriceFree = true {
        didSet {
            if isPriceFree {
                priceString = "0"
            }
        }
    }
    @Published var isAvailableComment = true
    @Published var postImage: UIImage? = nil
    
    @Published var priceString = "0" {
        didSet {
            if priceString.count > 5 {
                priceString = String(priceString.prefix(5))
            } else {
                if let price = Int(priceString) {
                    self.price = price
                } else {
                    self.price = 0
                }
            }
        }
    }
    @Published var price = 0
    @Published var soundData: Data? = nil
    
    var placeholder = "내용을 입력하세요"
    
    func createCommunityPost(onSuccess: @escaping () -> Void) {
        if !isLoading && validateData() {
            isLoading = true
            
            let request = CreateCommunityPostRequest(content: content, price: price, isAdult: isAdult, isCommentAvailable: isAvailableComment)
            var multipartData = [MultipartFormData]()
            
            let encoder = JSONEncoder()
            encoder.outputFormatting = .withoutEscapingSlashes
            let jsonData = try? encoder.encode(request)
            
            if let jsonData = jsonData {
                if let postImage = postImage, let imageData = postImage.jpegData(compressionQuality: 0.8) {
                    multipartData.append(
                        MultipartFormData(
                            provider: .data(imageData),
                            name: "postImage",
                            fileName: "\(UUID().uuidString)_\(Date().timeIntervalSince1970 * 1000).jpg",
                            mimeType: "image/*"
                        )
                    )
                }
                
                if let soundData = soundData {
                    multipartData.append(
                        MultipartFormData(
                            provider: .data(soundData),
                            name: "audioFile",
                            fileName: "\(UUID().uuidString)_\(Date().timeIntervalSince1970 * 1000).m4a",
                            mimeType: "audio/m4a"
                        )
                    )
                }
                
                multipartData.append(MultipartFormData(provider: .data(jsonData), name: "request"))
                
                repository
                    .createCommunityPost(parameters: multipartData)
                    .sink { result in
                        switch result {
                        case .finished:
                            DEBUG_LOG("finish")
                        case .failure(let error):
                            ERROR_LOG(error.localizedDescription)
                        }
                    } receiveValue: { [unowned self] response in
                        self.isLoading = false
                        let responseData = response.data
                        
                        do {
                            let jsonDecoder = JSONDecoder()
                            let decoded = try jsonDecoder.decode(ApiResponseWithoutData.self, from: responseData)
                            
                            if decoded.success {
                                self.errorMessage = "게시물이 등록되었습니다."
                                self.isShowPopup = true
                                
                                DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
                                    onSuccess()
                                }
                            } else {
                                if let message = decoded.message {
                                    self.errorMessage = message
                                } else {
                                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                                }
                                
                                self.isShowPopup = true
                            }
                        } catch {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                            self.isShowPopup = true
                        }
                    }
                    .store(in: &subscription)
            } else {
                self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                self.isShowPopup = true
                self.isLoading = false
            }
        }
    }
    
    private func validateData() -> Bool {
        if content.trimmingCharacters(in: .whitespacesAndNewlines).isEmpty || content.count < 5 {
            errorMessage = "내용을 5자 이상 입력해 주세요."
            isShowPopup = true
            return false
        }
        
        if !isPriceFree && price < 5 {
            errorMessage = "최소금액은 5캔 입니다."
            isShowPopup = true
            return false
        }
        
        return true
    }
}
